package xhttp3.internal.framed;

import java.io.Closeable;
import java.io.IOException;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.SynchronousQueue;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import xhttp3.Protocol;
import xhttp3.internal.NamedRunnable;
import xhttp3.internal.Util;
import xhttp3.internal.platform.Platform;
import xio.Buffer;
import xio.BufferedSink;
import xio.BufferedSource;
import xio.ByteString;
import xio.Okio;

public final class FramedConnection implements Closeable {
   private static final ExecutorService executor;
   private static final int OKHTTP_CLIENT_WINDOW_SIZE = 16777216;
   // $FF: synthetic field
   static final boolean $assertionsDisabled = FramedConnection.class.desiredAssertionStatus() ^ true;
   final Protocol protocol;
   final boolean client;
   private final Listener listener;
   private final Map<Integer, FramedStream> streams;
   private final String hostname;
   private int lastGoodStreamId;
   private int nextStreamId;
   private boolean shutdown;
   private final ExecutorService pushExecutor;
   private Map<Integer, Ping> pings;
   private final PushObserver pushObserver;
   private int nextPingId;
   long unacknowledgedBytesRead;
   long bytesLeftInWriteWindow;
   Settings okHttpSettings;
   final Settings peerSettings;
   private boolean receivedInitialPeerSettings;
   final Variant variant;
   final Socket socket;
   final FrameWriter frameWriter;
   final Reader readerRunnable;
   private final Set<Integer> currentPushRequests;

   private FramedConnection(Builder builder) {
      this.streams = new HashMap();
      this.unacknowledgedBytesRead = 0L;
      this.okHttpSettings = new Settings();
      Settings var2;
      Settings var10012 = var2 = new Settings;
      var10012.<init>();
      this.peerSettings = var10012;
      this.receivedInitialPeerSettings = false;
      this.currentPushRequests = new LinkedHashSet();
      Protocol var3;
      this.protocol = var3 = builder.protocol;
      this.pushObserver = builder.pushObserver;
      boolean var4;
      this.client = var4 = builder.client;
      this.listener = builder.listener;
      byte var5;
      if (builder.client) {
         var5 = 1;
      } else {
         var5 = 2;
      }

      this.nextStreamId = var5;
      if (builder.client && var3 == Protocol.HTTP_2) {
         this.nextStreamId += 2;
      }

      if (builder.client) {
         var5 = 1;
      } else {
         var5 = 2;
      }

      this.nextPingId = var5;
      if (builder.client) {
         this.okHttpSettings.set(7, 0, 16777216);
      }

      String var10;
      this.hostname = var10 = builder.hostname;
      if (var3 == Protocol.HTTP_2) {
         this.variant = new Http2();
         TimeUnit var8 = TimeUnit.SECONDS;
         LinkedBlockingQueue var6;
         var6 = new LinkedBlockingQueue.<init>();
         Object[] var7;
         (var7 = new Object[1])[0] = var10;
         ThreadFactory var11 = Util.threadFactory(Util.format("OkHttp %s Push Observer", var7), true);
         this.pushExecutor = new ThreadPoolExecutor(0, 1, 60L, var8, var6, var11);
         var2.set(7, 0, 65535);
         var2.set(5, 0, 16384);
      } else {
         if (var3 != Protocol.SPDY_3) {
            throw new AssertionError(var3);
         }

         Spdy3 var9;
         var9 = new Spdy3.<init>();
         this.variant = var9;
         this.pushExecutor = null;
      }

      this.bytesLeftInWriteWindow = (long)var2.getInitialWindowSize(65536);
      this.socket = builder.socket;
      this.frameWriter = this.variant.newWriter(builder.sink, var4);
      this.readerRunnable = new Reader(this.variant.newReader(builder.source, var4));
   }

   private FramedStream newStream(int param1, List<Header> param2, boolean param3, boolean param4) {
      // $FF: Couldn't be decompiled
   }

   private void writePingLater(final boolean reply, final int payload1, final int payload2, final Ping ping) {
      ExecutorService var10000 = executor;
      Object[] var5;
      Object[] var10004 = var5 = new Object[3];
      var5[0] = this.hostname;
      var5[1] = payload1;
      var10004[2] = payload2;
      var10000.execute(new NamedRunnable("OkHttp %s ping %08x%08x", var5) {
         public void execute() {
            <undefinedtype> var10;
            FramedConnection var10000;
            boolean var10001;
            <undefinedtype> var10002;
            <undefinedtype> var10003;
            boolean var10004;
            try {
               var10000 = FramedConnection.this;
               var10 = this;
               var10002 = this;
               var10003 = this;
               var10004 = reply;
            } catch (IOException var8) {
               var10001 = false;
               return;
            }

            boolean var9 = var10004;

            int var13;
            try {
               var13 = payload1;
            } catch (IOException var7) {
               var10001 = false;
               return;
            }

            int var1 = var13;

            int var11;
            try {
               var11 = payload2;
            } catch (IOException var6) {
               var10001 = false;
               return;
            }

            int var2 = var11;

            Ping var12;
            try {
               var12 = ping;
            } catch (IOException var5) {
               var10001 = false;
               return;
            }

            Ping var3 = var12;

            try {
               var10000.writePing(var9, var1, var2, var3);
            } catch (IOException var4) {
               var10001 = false;
            }

         }
      });
   }

   private void writePing(boolean reply, int payload1, int payload2, Ping ping) {
      FrameWriter var5;
      synchronized(var5 = this.frameWriter){}
      Throwable var10000;
      boolean var10001;
      if (ping != null) {
         try {
            ping.send();
         } catch (Throwable var11) {
            var10000 = var11;
            var10001 = false;
            throw var10000;
         }
      }

      try {
         this.frameWriter.ping(reply, payload1, payload2);
      } catch (Throwable var10) {
         var10000 = var10;
         var10001 = false;
         throw var10000;
      }
   }

   private synchronized Ping removePing(int id) {
      Map var2;
      return (var2 = this.pings) != null ? (Ping)var2.remove(id) : null;
   }

   private void close(ErrorCode connectionCode, ErrorCode streamCode) {
      if (!$assertionsDisabled && Thread.holdsLock(this)) {
         throw new AssertionError();
      } else {
         FramedConnection var10000 = this;
         ErrorCode var10001 = connectionCode;
         IOException connectionCode1 = null;

         try {
            var10000.shutdown(var10001);
         } catch (IOException var85) {
            connectionCode1 = var85;
         }

         var10000 = this;
         FramedStream[] var3 = null;
         Ping[] var4 = null;
         synchronized(this){}

         Throwable var102;
         boolean var103;
         boolean var104;
         try {
            var103 = var10000.streams.isEmpty();
         } catch (Throwable var95) {
            var102 = var95;
            var104 = false;
            throw var102;
         }

         if (!var103) {
            FramedStream[] var105;
            try {
               var10000 = this;
               var105 = (FramedStream[])this.streams.values().toArray(new FramedStream[this.streams.size()]);
            } catch (Throwable var94) {
               var102 = var94;
               var104 = false;
               throw var102;
            }

            var3 = var105;

            try {
               var10000.streams.clear();
            } catch (Throwable var93) {
               var102 = var93;
               var104 = false;
               throw var102;
            }
         }

         Map var106;
         try {
            var106 = this.pings;
         } catch (Throwable var92) {
            var102 = var92;
            var104 = false;
            throw var102;
         }

         Map var5 = var106;
         if (var106 != null) {
            Ping[] var107;
            try {
               var10000 = this;
               var107 = (Ping[])var5.values().toArray(new Ping[this.pings.size()]);
            } catch (Throwable var91) {
               var102 = var91;
               var104 = false;
               throw var102;
            }

            var4 = var107;

            try {
               var10000.pings = null;
            } catch (Throwable var90) {
               var102 = var90;
               var104 = false;
               throw var102;
            }
         }

         FramedStream[] var108;
         try {
            var108 = var3;
         } catch (Throwable var89) {
            var102 = var89;
            var104 = false;
            throw var102;
         }

         if (var108 != null) {
            int var101 = var3.length;

            IOException var7;
            for(int var6 = 0; var6 < var101; connectionCode1 = var7) {
               FramedStream var109 = var3[var6];

               label926: {
                  try {
                     var109.close(streamCode);
                  } catch (IOException var88) {
                     var7 = var88;
                     if (connectionCode1 != null) {
                        break label926;
                     }
                  }

                  var7 = connectionCode1;
               }

               ++var6;
            }
         }

         if (var4 != null) {
            int streamCode1 = var4.length;

            for(int var100 = 0; var100 < streamCode1; ++var100) {
               var4[var100].cancel();
            }
         }

         IOException streamCode2;
         label910: {
            try {
               this.frameWriter.close();
            } catch (IOException var87) {
               streamCode2 = var87;
               if (connectionCode1 == null) {
                  break label910;
               }
            }

            streamCode2 = connectionCode1;
         }

         IOException var96;
         label905: {
            try {
               this.socket.close();
            } catch (IOException var86) {
               var96 = var86;
               break label905;
            }

            var96 = streamCode2;
         }

         if (var96 != null) {
            throw var96;
         }
      }
   }

   private boolean pushedStream(int streamId) {
      return this.protocol == Protocol.HTTP_2 && streamId != 0 && (streamId & 1) == 0;
   }

   private void pushRequestLater(final int streamId, final List<Header> requestHeaders) {
      FramedConnection var10000 = this;
      synchronized(this){}

      boolean var10001;
      Throwable var24;
      boolean var25;
      try {
         var25 = var10000.currentPushRequests.contains(streamId);
      } catch (Throwable var23) {
         var24 = var23;
         var10001 = false;
         throw var24;
      }

      if (var25) {
         try {
            this.writeSynResetLater(streamId, ErrorCode.PROTOCOL_ERROR);
         } catch (Throwable var20) {
            var24 = var20;
            var10001 = false;
            throw var24;
         }
      } else {
         FramedConnection var26;
         try {
            var10000 = this;
            var26 = this;
            this.currentPushRequests.add(streamId);
         } catch (Throwable var22) {
            var24 = var22;
            var10001 = false;
            throw var24;
         }

         try {
            ;
         } catch (Throwable var21) {
            var24 = var21;
            var10001 = false;
            throw var24;
         }

         ExecutorService var27 = var10000.pushExecutor;
         Object[] var3;
         Object[] var10004 = var3 = new Object[2];
         var3[0] = this.hostname;
         var10004[1] = streamId;
         var27.execute(new NamedRunnable("OkHttp %s Push Request[%s]", var3) {
            public void execute() {
               // $FF: Couldn't be decompiled
            }
         });
      }
   }

   private void pushHeadersLater(final int streamId, final List<Header> requestHeaders, final boolean inFinished) {
      ExecutorService var10000 = this.pushExecutor;
      Object[] var4;
      Object[] var10004 = var4 = new Object[2];
      var4[0] = this.hostname;
      var10004[1] = streamId;
      var10000.execute(new NamedRunnable("OkHttp %s Push Headers[%s]", var4) {
         public void execute() {
            // $FF: Couldn't be decompiled
         }
      });
   }

   private void pushDataLater(final int streamId, BufferedSource source, final int byteCount, final boolean inFinished) {
      BufferedSource var10000 = source;
      BufferedSource var10001 = source;
      final Buffer source1;
      source1 = new Buffer.<init>();
      long var5;
      var10001.require(var5 = (long)byteCount);
      var10000.read(source1, var5);
      if (source1.size() == var5) {
         ExecutorService var9 = this.pushExecutor;
         Object[] var8;
         Object[] var10004 = var8 = new Object[2];
         var8[0] = this.hostname;
         var10004[1] = streamId;
         var9.execute(new NamedRunnable("OkHttp %s Push Data[%s]", var8) {
            public void execute() {
               // $FF: Couldn't be decompiled
            }
         });
      } else {
         throw new IOException(source1.size() + " != " + byteCount);
      }
   }

   private void pushResetLater(final int streamId, final ErrorCode errorCode) {
      ExecutorService var10000 = this.pushExecutor;
      Object[] var3;
      Object[] var10004 = var3 = new Object[2];
      var3[0] = this.hostname;
      var10004[1] = streamId;
      var10000.execute(new NamedRunnable("OkHttp %s Push Reset[%s]", var3) {
         public void execute() {
            PushObserver var10001 = FramedConnection.this.pushObserver;
            int var1 = streamId;
            var10001.onReset(var1, errorCode);
            FramedConnection var8;
            FramedConnection var10000 = var8 = FramedConnection.this;
            <undefinedtype> var10 = this;
            synchronized(var8){}

            Throwable var9;
            boolean var11;
            try {
               FramedConnection.this.currentPushRequests.remove(streamId);
            } catch (Throwable var7) {
               var9 = var7;
               var11 = false;
               throw var9;
            }

            try {
               ;
            } catch (Throwable var6) {
               var9 = var6;
               var11 = false;
               throw var9;
            }
         }
      });
   }

   // $FF: synthetic method
   FramedConnection(Builder x0, Object x1) {
      this(x0);
   }

   // $FF: synthetic method
   static void access$1200(FramedConnection x0, ErrorCode x1, ErrorCode x2) {
      x0.close(x1, x2);
   }

   static {
      TimeUnit var0 = TimeUnit.SECONDS;
      SynchronousQueue var1;
      var1 = new SynchronousQueue.<init>();
      ThreadFactory var2 = Util.threadFactory("OkHttp FramedConnection", true);
      executor = new ThreadPoolExecutor(0, Integer.MAX_VALUE, 60L, var0, var1, var2);
   }

   public Protocol getProtocol() {
      return this.protocol;
   }

   public synchronized int openStreamCount() {
      return this.streams.size();
   }

   synchronized FramedStream getStream(int id) {
      return (FramedStream)this.streams.get(id);
   }

   synchronized FramedStream removeStream(int streamId) {
      FramedStream var10000 = (FramedStream)this.streams.remove(streamId);
      this.notifyAll();
      return var10000;
   }

   public synchronized int maxConcurrentStreams() {
      return this.peerSettings.getMaxConcurrentStreams(Integer.MAX_VALUE);
   }

   public FramedStream pushStream(int associatedStreamId, List<Header> requestHeaders, boolean out) {
      if (!this.client) {
         if (this.protocol == Protocol.HTTP_2) {
            return this.newStream(associatedStreamId, requestHeaders, out, false);
         } else {
            throw new IllegalStateException("protocol != HTTP_2");
         }
      } else {
         throw new IllegalStateException("Client cannot push requests.");
      }
   }

   public FramedStream newStream(List<Header> requestHeaders, boolean out, boolean in) {
      return this.newStream(0, requestHeaders, out, in);
   }

   void writeSynReply(int streamId, boolean outFinished, List<Header> alternating) {
      this.frameWriter.synReply(outFinished, streamId, alternating);
   }

   public void writeData(int param1, boolean param2, Buffer param3, long param4) {
      // $FF: Couldn't be decompiled
   }

   void addBytesToWriteWindow(long delta) {
      this.bytesLeftInWriteWindow += delta;
      if (delta > 0L) {
         this.notifyAll();
      }

   }

   void writeSynResetLater(final int streamId, final ErrorCode errorCode) {
      ExecutorService var10000 = executor;
      Object[] var3;
      Object[] var10004 = var3 = new Object[2];
      var3[0] = this.hostname;
      var10004[1] = streamId;
      var10000.submit(new NamedRunnable("OkHttp %s stream %d", var3) {
         public void execute() {
            // $FF: Couldn't be decompiled
         }
      });
   }

   void writeSynReset(int streamId, ErrorCode statusCode) {
      this.frameWriter.rstStream(streamId, statusCode);
   }

   void writeWindowUpdateLater(final int streamId, final long unacknowledgedBytesRead) {
      ExecutorService var10000 = executor;
      Object[] var4;
      Object[] var10004 = var4 = new Object[2];
      var4[0] = this.hostname;
      var10004[1] = streamId;
      var10000.execute(new NamedRunnable("OkHttp Window Update %s stream %d", var4) {
         public void execute() {
            FrameWriter var10000;
            boolean var10001;
            int var10002;
            <undefinedtype> var7;
            try {
               var10000 = FramedConnection.this.frameWriter;
               var7 = this;
               var10002 = streamId;
            } catch (IOException var5) {
               var10001 = false;
               return;
            }

            int var6 = var10002;

            long var8;
            try {
               var8 = unacknowledgedBytesRead;
            } catch (IOException var4) {
               var10001 = false;
               return;
            }

            long var1 = var8;

            try {
               var10000.windowUpdate(var6, var1);
            } catch (IOException var3) {
               var10001 = false;
            }

         }
      });
   }

   public Ping ping() {
      FramedConnection var10000 = this;
      Ping var1;
      var1 = new Ping.<init>();
      synchronized(this){}

      boolean var10001;
      Throwable var59;
      boolean var60;
      try {
         var60 = var10000.shutdown;
      } catch (Throwable var58) {
         var59 = var58;
         var10001 = false;
         throw var59;
      }

      if (!var60) {
         int var10002;
         FramedConnection var61;
         try {
            var10000 = this;
            var61 = this;
            var10002 = this.nextPingId;
         } catch (Throwable var56) {
            var59 = var56;
            var10001 = false;
            throw var59;
         }

         int var2 = var10002;
         var10002 += 2;

         Map var62;
         try {
            var61.nextPingId = var10002;
            var62 = var10000.pings;
         } catch (Throwable var55) {
            var59 = var55;
            var10001 = false;
            throw var59;
         }

         if (var62 == null) {
            try {
               this.pings = new HashMap();
            } catch (Throwable var54) {
               var59 = var54;
               var10001 = false;
               throw var59;
            }
         }

         Ping var64;
         FramedConnection var63;
         try {
            var64 = var1;
            var61 = this;
            var63 = this;
            this.pings.put(var2, var1);
         } catch (Throwable var53) {
            var59 = var53;
            var10001 = false;
            throw var59;
         }

         try {
            ;
         } catch (Throwable var52) {
            var59 = var52;
            var10001 = false;
            throw var59;
         }

         var61.writePing(false, var2, 1330343787, var1);
         return var64;
      } else {
         try {
            throw new IOException("shutdown");
         } catch (Throwable var57) {
            var59 = var57;
            var10001 = false;
            throw var59;
         }
      }
   }

   public void flush() {
      this.frameWriter.flush();
   }

   public void shutdown(ErrorCode param1) {
      // $FF: Couldn't be decompiled
   }

   public void close() {
      this.close(ErrorCode.NO_ERROR, ErrorCode.CANCEL);
   }

   public void start() {
      this.start(true);
   }

   void start(boolean sendConnectionPreface) {
      if (sendConnectionPreface) {
         this.frameWriter.connectionPreface();
         this.frameWriter.settings(this.okHttpSettings);
         int sendConnectionPreface1;
         if ((sendConnectionPreface1 = this.okHttpSettings.getInitialWindowSize(65536)) != 65536) {
            long var2 = (long)(sendConnectionPreface1 - 65536);
            this.frameWriter.windowUpdate(0, var2);
         }
      }

      (new Thread(this.readerRunnable)).start();
   }

   public void setSettings(Settings param1) {
      // $FF: Couldn't be decompiled
   }

   public abstract static class Listener {
      public static final Listener REFUSE_INCOMING_STREAMS = new Listener() {
         public void onStream(FramedStream stream) {
            stream.close(ErrorCode.REFUSED_STREAM);
         }
      };

      public abstract void onStream(FramedStream var1);

      public void onSettings(FramedConnection connection) {
      }
   }

   class Reader extends NamedRunnable implements FrameReader.Handler {
      final FrameReader frameReader;

      private Reader(FrameReader frameReader) {
         Reader var10000 = this;
         Reader var10001 = this;
         Object[] var3;
         (var3 = new Object[1])[0] = FramedConnection.this.hostname;
         var10001.<init>("OkHttp %s", var3);
         var10000.frameReader = frameReader;
      }

      private void applyAndAckSettings(final Settings peerSettings) {
         ExecutorService var10000 = FramedConnection.executor;
         Object[] var2;
         (var2 = new Object[1])[0] = FramedConnection.this.hostname;
         var10000.execute(new NamedRunnable("OkHttp %s ACK Settings", var2) {
            public void execute() {
               try {
                  FramedConnection.this.frameWriter.applyAndAckSettings(peerSettings);
               } catch (IOException var1) {
               }

            }
         });
      }

      // $FF: synthetic method
      Reader(FrameReader x1, Object x2) {
         this(x1);
      }

      protected void execute() {
         // $FF: Couldn't be decompiled
      }

      public void data(boolean inFinished, int streamId, BufferedSource source, int length) {
         if (FramedConnection.this.pushedStream(streamId)) {
            FramedConnection.this.pushDataLater(streamId, source, length, inFinished);
         } else {
            FramedStream var5;
            if ((var5 = FramedConnection.this.getStream(streamId)) == null) {
               FramedConnection.this.writeSynResetLater(streamId, ErrorCode.INVALID_STREAM);
               source.skip((long)length);
            } else {
               var5.receiveData(source, length);
               if (inFinished) {
                  var5.receiveFin();
               }

            }
         }
      }

      public void headers(boolean outFinished, boolean inFinished, int streamId, int associatedStreamId, List<Header> headerBlock, HeadersMode headersMode) {
         if (FramedConnection.this.pushedStream(streamId)) {
            FramedConnection.this.pushHeadersLater(streamId, headerBlock, inFinished);
         } else {
            Reader var10000 = this;
            FramedConnection associatedStreamId1;
            synchronized(associatedStreamId1 = FramedConnection.this){}

            boolean var10001;
            Throwable var318;
            boolean var319;
            try {
               var319 = FramedConnection.this.shutdown;
            } catch (Throwable var313) {
               var318 = var313;
               var10001 = false;
               throw var318;
            }

            if (var319) {
               try {
                  ;
               } catch (Throwable var297) {
                  var318 = var297;
                  var10001 = false;
                  throw var318;
               }
            } else {
               FramedStream var320;
               try {
                  var320 = FramedConnection.this.getStream(streamId);
               } catch (Throwable var312) {
                  var318 = var312;
                  var10001 = false;
                  throw var318;
               }

               FramedStream var7 = var320;
               if (var320 == null) {
                  try {
                     var319 = headersMode.failIfStreamAbsent();
                  } catch (Throwable var310) {
                     var318 = var310;
                     var10001 = false;
                     throw var318;
                  }

                  if (var319) {
                     try {
                        FramedConnection.this.writeSynResetLater(streamId, ErrorCode.INVALID_STREAM);
                     } catch (Throwable var298) {
                        var318 = var298;
                        var10001 = false;
                        throw var318;
                     }
                  } else {
                     int var322;
                     int var323;
                     try {
                        var323 = streamId;
                        var322 = FramedConnection.this.lastGoodStreamId;
                     } catch (Throwable var309) {
                        var318 = var309;
                        var10001 = false;
                        throw var318;
                     }

                     if (var323 <= var322) {
                        try {
                           ;
                        } catch (Throwable var299) {
                           var318 = var299;
                           var10001 = false;
                           throw var318;
                        }
                     } else {
                        try {
                           var323 = streamId % 2;
                           var322 = FramedConnection.this.nextStreamId % 2;
                        } catch (Throwable var308) {
                           var318 = var308;
                           var10001 = false;
                           throw var318;
                        }

                        if (var323 == var322) {
                           try {
                              ;
                           } catch (Throwable var300) {
                              var318 = var300;
                              var10001 = false;
                              throw var318;
                           }
                        } else {
                           Reader var10002;
                           FramedStream var10003;
                           Reader var325;
                           FramedConnection var326;
                           try {
                              var326 = associatedStreamId1;
                              var325 = this;
                              var10002 = this;
                              var10003 = new FramedStream;
                           } catch (Throwable var307) {
                              var318 = var307;
                              var10001 = false;
                              throw var318;
                           }

                           FramedStream headersMode1 = var10003;

                           try {
                              var10003.<init>(streamId, FramedConnection.this, outFinished, inFinished, headerBlock);
                              FramedConnection.this.lastGoodStreamId = streamId;
                           } catch (Throwable var306) {
                              var318 = var306;
                              var10001 = false;
                              throw var318;
                           }

                           try {
                              FramedConnection.this.streams.put(streamId, headersMode1);
                           } catch (Throwable var305) {
                              var318 = var305;
                              var10001 = false;
                              throw var318;
                           }

                           NamedRunnable var321;
                           ExecutorService var327;
                           try {
                              var327 = FramedConnection.executor;
                              var321 = new NamedRunnable() {
                                 // $FF: synthetic field
                                 final FramedStream val$newStream;

                                 {
                                    this.val$newStream = var4;
                                 }

                                 public void execute() {
                                    try {
                                       FramedConnection.this.listener.onStream(this.val$newStream);
                                    } catch (IOException var3) {
                                       <undefinedtype> var10000 = this;
                                       Platform.get().log(4, "FramedConnection.Listener failure for " + FramedConnection.this.hostname, var3);

                                       try {
                                          var10000.val$newStream.close(ErrorCode.PROTOCOL_ERROR);
                                       } catch (IOException var2) {
                                       }
                                    }

                                 }
                              };
                           } catch (Throwable var304) {
                              var318 = var304;
                              var10001 = false;
                              throw var318;
                           }

                           NamedRunnable var324 = var321;
                           Reader var10004 = this;
                           String var10005 = "OkHttp %s stream %d";

                           Object[] var10006;
                           try {
                              var10006 = new Object[2];
                           } catch (Throwable var303) {
                              var318 = var303;
                              var10001 = false;
                              throw var318;
                           }

                           Object[] outFinished1 = var10006;
                           FramedStream var10007 = headersMode1;
                           Object[] var10008 = outFinished1;
                           int var10009 = streamId;
                           Object[] var10010 = outFinished1;
                           byte var10011 = 0;

                           try {
                              var10010[var10011] = FramedConnection.this.hostname;
                           } catch (Throwable var302) {
                              var318 = var302;
                              var10001 = false;
                              throw var318;
                           }

                           byte var314 = 1;

                           try {
                              var10008[var314] = var10009;
                              var324.<init>(var10005, var10006, var10007);
                              var327.execute(var321);
                           } catch (Throwable var301) {
                              var318 = var301;
                              var10001 = false;
                              throw var318;
                           }
                        }
                     }
                  }
               } else {
                  HeadersMode var328;
                  try {
                     var328 = headersMode;
                  } catch (Throwable var311) {
                     var318 = var311;
                     var10001 = false;
                     throw var318;
                  }

                  if (var328.failIfStreamPresent()) {
                     var7.closeLater(ErrorCode.PROTOCOL_ERROR);
                     FramedConnection.this.removeStream(streamId);
                  } else {
                     var7.receiveHeaders(headerBlock, headersMode);
                     if (inFinished) {
                        var7.receiveFin();
                     }

                  }
               }
            }
         }
      }

      public void rstStream(int streamId, ErrorCode errorCode) {
         if (FramedConnection.this.pushedStream(streamId)) {
            FramedConnection.this.pushResetLater(streamId, errorCode);
         } else {
            FramedStream var3;
            if ((var3 = FramedConnection.this.removeStream(streamId)) != null) {
               var3.receiveRstStream(errorCode);
            }

         }
      }

      public void settings(boolean clearPrevious, Settings newSettings) {
         Reader var10001 = this;
         long var3 = 0L;
         FramedStream[] clearPrevious1 = null;
         FramedConnection var5;
         synchronized(var5 = FramedConnection.this){}

         Throwable var180;
         boolean var181;
         int var183;
         try {
            var183 = FramedConnection.this.peerSettings.getInitialWindowSize(65536);
         } catch (Throwable var175) {
            var180 = var175;
            var181 = false;
            throw var180;
         }

         int var6 = var183;
         if (clearPrevious) {
            try {
               FramedConnection.this.peerSettings.clear();
            } catch (Throwable var174) {
               var180 = var174;
               var181 = false;
               throw var180;
            }
         }

         Protocol var182;
         Protocol var186;
         try {
            FramedConnection.this.peerSettings.merge(newSettings);
            var182 = FramedConnection.this.getProtocol();
            var186 = Protocol.HTTP_2;
         } catch (Throwable var173) {
            var180 = var173;
            var181 = false;
            throw var180;
         }

         if (var182 == var186) {
            try {
               this.applyAndAckSettings(newSettings);
            } catch (Throwable var172) {
               var180 = var172;
               var181 = false;
               throw var180;
            }
         }

         int var184;
         try {
            var184 = FramedConnection.this.peerSettings.getInitialWindowSize(65536);
         } catch (Throwable var171) {
            var180 = var171;
            var181 = false;
            throw var180;
         }

         int newSettings1 = var184;
         FramedStream[] var188;
         if (var184 != -1 && newSettings1 != var6) {
            Reader var187 = this;
            var3 = (long)(newSettings1 - var6);

            boolean var10000;
            try {
               var10000 = FramedConnection.this.receivedInitialPeerSettings;
            } catch (Throwable var170) {
               var180 = var170;
               var181 = false;
               throw var180;
            }

            if (!var10000) {
               try {
                  FramedConnection.this.addBytesToWriteWindow(var3);
                  FramedConnection.this.receivedInitialPeerSettings = true;
               } catch (Throwable var169) {
                  var180 = var169;
                  var181 = false;
                  throw var180;
               }
            }

            try {
               var10000 = FramedConnection.this.streams.isEmpty();
            } catch (Throwable var168) {
               var180 = var168;
               var181 = false;
               throw var180;
            }

            if (!var10000) {
               try {
                  var188 = (FramedStream[])FramedConnection.this.streams.values().toArray(new FramedStream[FramedConnection.this.streams.size()]);
               } catch (Throwable var167) {
                  var180 = var167;
                  var181 = false;
                  throw var180;
               }

               clearPrevious1 = var188;
            }
         }

         ExecutorService var10002;
         NamedRunnable var10003;
         FramedConnection var189;
         try {
            var188 = clearPrevious1;
            var189 = var5;
            var10002 = FramedConnection.executor;
            var10003 = new NamedRunnable() {
               public void execute() {
                  FramedConnection.this.listener.onSettings(FramedConnection.this);
               }
            };
         } catch (Throwable var166) {
            var180 = var166;
            var181 = false;
            throw var180;
         }

         NamedRunnable var10004 = var10003;
         Reader var10005 = this;
         String var10006 = "OkHttp %s settings";

         Object[] var10007;
         try {
            var10007 = new Object[1];
         } catch (Throwable var165) {
            var180 = var165;
            var181 = false;
            throw var180;
         }

         Object[] var10008 = var10007;
         byte var10009 = 0;

         try {
            var10008[var10009] = FramedConnection.this.hostname;
            var10004.<init>(var10006, var10007);
            var10002.execute(var10003);
         } catch (Throwable var164) {
            var180 = var164;
            var181 = false;
            throw var180;
         }

         if (var188 != null && var3 != 0L) {
            int var176 = clearPrevious1.length;

            for(newSettings1 = 0; newSettings1 < var176; ++newSettings1) {
               FramedStream var191 = clearPrevious1[newSettings1];
               FramedStream var179;
               FramedStream var190 = var179 = clearPrevious1[newSettings1];
               long var185 = var3;
               synchronized(var179) {
                  var190.addBytesToWriteWindow(var185);
               }
            }
         }

      }

      public void ackSettings() {
      }

      public void ping(boolean reply, int payload1, int payload2) {
         if (reply) {
            Ping var4;
            if ((var4 = FramedConnection.this.removePing(payload1)) != null) {
               var4.receive();
            }
         } else {
            FramedConnection.this.writePingLater(true, payload1, payload2, (Ping)null);
         }

      }

      public void goAway(int lastGoodStreamId, ErrorCode errorCode, ByteString debugData) {
         Reader var10000 = this;
         debugData.size();
         FramedConnection errorCode1;
         synchronized(errorCode1 = FramedConnection.this){}

         boolean var10001;
         Throwable var21;
         FramedStream[] var22;
         try {
            var22 = (FramedStream[])FramedConnection.this.streams.values().toArray(new FramedStream[FramedConnection.this.streams.size()]);
         } catch (Throwable var17) {
            var21 = var17;
            var10001 = false;
            throw var21;
         }

         FramedStream[] debugData1 = var22;

         FramedConnection var23;
         try {
            var23 = errorCode1;
            FramedConnection.this.shutdown = true;
         } catch (Throwable var16) {
            var21 = var16;
            var10001 = false;
            throw var21;
         }

         try {
            ;
         } catch (Throwable var15) {
            var21 = var15;
            var10001 = false;
            throw var21;
         }

         int errorCode2 = var22.length;

         for(int var4 = 0; var4 < errorCode2; ++var4) {
            FramedStream var5;
            if ((var5 = debugData1[var4]).getId() > lastGoodStreamId && var5.isLocallyInitiated()) {
               var5.receiveRstStream(ErrorCode.REFUSED_STREAM);
               FramedConnection.this.removeStream(var5.getId());
            }
         }

      }

      public void windowUpdate(int streamId, long windowSizeIncrement) {
         if (streamId == 0) {
            FramedConnection var4;
            FramedConnection var15 = var4 = FramedConnection.this;
            Reader var17 = this;
            synchronized(var4){}

            Throwable var16;
            boolean var18;
            FramedConnection var19;
            long var10003;
            FramedConnection var20;
            try {
               var19 = FramedConnection.this;
               var20 = var19;
               var10003 = var19.bytesLeftInWriteWindow;
            } catch (Throwable var13) {
               var16 = var13;
               var18 = false;
               throw var16;
            }

            var10003 += windowSizeIncrement;

            try {
               var20.bytesLeftInWriteWindow = var10003;
               var19.notifyAll();
               return;
            } catch (Throwable var12) {
               var16 = var12;
               var18 = false;
               throw var16;
            }
         } else {
            FramedStream var14;
            if ((var14 = FramedConnection.this.getStream(streamId)) != null) {
               FramedStream var10000 = var14;
               FramedStream var10001 = var14;
               long var10002 = windowSizeIncrement;
               synchronized(var14) {
                  var10001.addBytesToWriteWindow(var10002);
               }
            }
         }

      }

      public void priority(int streamId, int streamDependency, int weight, boolean exclusive) {
      }

      public void pushPromise(int streamId, int promisedStreamId, List<Header> requestHeaders) {
         FramedConnection.this.pushRequestLater(promisedStreamId, requestHeaders);
      }

      public void alternateService(int streamId, String origin, ByteString protocol, String host, int port, long maxAge) {
      }
   }

   public static class Builder {
      private Socket socket;
      private String hostname;
      private BufferedSource source;
      private BufferedSink sink;
      private Listener listener;
      private Protocol protocol;
      private PushObserver pushObserver;
      private boolean client;

      public Builder(boolean client) {
         this.listener = FramedConnection.Listener.REFUSE_INCOMING_STREAMS;
         this.protocol = Protocol.SPDY_3;
         this.pushObserver = PushObserver.CANCEL;
         this.client = client;
      }

      public Builder socket(Socket socket) {
         Builder var10000 = this;
         Socket var10001 = socket;
         Socket var10002 = socket;
         String var3 = ((InetSocketAddress)socket.getRemoteSocketAddress()).getHostName();
         BufferedSource socket1 = Okio.buffer(Okio.source(socket));
         BufferedSink var2 = Okio.buffer(Okio.sink(var10002));
         return var10000.socket(var10001, var3, socket1, var2);
      }

      public Builder socket(Socket socket, String hostname, BufferedSource source, BufferedSink sink) {
         this.socket = socket;
         this.hostname = hostname;
         this.source = source;
         this.sink = sink;
         return this;
      }

      public Builder listener(Listener listener) {
         this.listener = listener;
         return this;
      }

      public Builder protocol(Protocol protocol) {
         this.protocol = protocol;
         return this;
      }

      public Builder pushObserver(PushObserver pushObserver) {
         this.pushObserver = pushObserver;
         return this;
      }

      public FramedConnection build() {
         return new FramedConnection(this);
      }
   }
}
